--MRengineBrake 
--Engine Brake for MR Trucks with MR manual transmission only without manual clutch.
--	Credits:
--Based on original enginebrake.lua by TwistedGA & Looseterror
--MR edit westmorgan
--1/25/2014 ver. 2 Final


engineBrake = {};



function engineBrake.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function engineBrake:load(xmlFile)
	local engineBrakeSound = getXMLString(xmlFile, "vehicle.engineBrakeSound#file");
	if engineBrakeSound ~= nil and engineBrakeSound ~= "" then
		engineBrakeSound = Utils.getFilename(engineBrakeSound, self.baseDirectory);
		self.engineBrakeSound = createSample("engineBrakeSound");
		loadSample(self.engineBrakeSound, engineBrakeSound, false);
		self.engineBrakeSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.engineBrakeSound#pitchOffset"), 0);
		self.engineBrakeSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.engineBrakeSound#pitchScale"), 0.05);
		self.engineBrakeSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.engineBrakeSound#pitchMax"), 2.0);
		self.engineBrakeSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.engineBrakeSound#volume"), 10.0);
		self.engineBrakeSoundEnabled = false;
	end;
-- Configure Variables
	self.engineBrake = false;
	self.ebHiLo = true;
	self.ebRealMotorBrakeFxBackup = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.realMotorBrakeFx"), 1);
	self.motorBrakeHigh = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorBrakeHigh"), 3);
	self.motorBrakeLow = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.motorBrakeLow"), 1.75);
end;

function engineBrake:update(dt)
	if self.isEntered and self.isMotorStarted then
			if InputBinding.hasEvent(InputBinding.ENGINEBRAKE) then		
				self.engineBrake = not self.engineBrake
			end;
			if InputBinding.hasEvent(InputBinding.EBHILO) then		
			self.ebHiLo = not self.ebHiLo
			end;
	end;		
-- Engine Brake
	if self.engineBrake then
	-- Set engineBrakeSound pitch based on Engine RPM		
		local currentEngineRpm = RealisticUtils.linearFx(self.realSoundEngineRevFx, self.mrFpMinRpm, self.mrFpMaxRpm);
		local alpha = 0.9; --0.9
		local roundPerMinute = self.lastRoundPerMinute*alpha + (1-alpha)*(currentEngineRpm-self.mrFpMinRpm);
			self.lastRoundPerMinute = roundPerMinute;
		local roundPerSecond = roundPerMinute / 130; --/60
		if self.engineBrakeSound ~= nil then
			setSamplePitch(self.engineBrakeSound, math.min(self.engineBrakeSoundPitchOffset + self.engineBrakeSoundPitchScale*math.abs(roundPerSecond), self.engineBrakeSoundPitchMax));
			setSampleVolume(self.engineBrakeSound, math.min(0.5 + 0.1*math.abs(roundPerSecond)), 10);
		end;
	--
		local input = 0;		
		if self.axisForward ~= nil then
			input = self.axisForward;
		end;
		if self.engineBrakeSound ~= nil then
			local enoughRpm = false;
			if currentEngineRpm > (self.mrFpMinRpm + (self.mrFpMinRpm * 0.1)) then
				enoughRpm = true;			 
			end;
			if self.engineBrakeSoundEnabled and input == -1 then
				stopSample(self.engineBrakeSound)
				self.engineBrakeSoundEnabled = false;
			end;
			-- Detect downshift			
			if InputBinding.hasEvent(InputBinding.MRFULLPOWERSHIFTGEARDOWN) then		
				stopSample(self.engineBrakeSound);
				self.engineBrakeSoundEnabled = false;
			end;	
			if input == -1 and enoughRpm then
				-- if we drive fast enough, we need to run the exhaustbrake the next time we release the throttle
				self.engineBrakeSoundEnabled = false;
			end;
			if input >= 0 and not self.engineBrakeSoundEnabled and enoughRpm and self.realGroundSpeed > 1 then
				-- play the exhaustbrake sound if we drive fast enough and release throttle
				if self:getIsActiveForSound() then
					playSample(self.engineBrakeSound, 1, self.engineBrakeSoundVolume, 0);
					setSamplePitch(self.engineBrakeSound, self.engineBrakeSoundPitchOffset);					
					if 	self.ebHiLo then
						self.realMotorBrakeFx = self.motorBrakeLow;
					else self.realMotorBrakeFx = self.motorBrakeHigh;
					end;
				end;
				self.engineBrakeSoundEnabled = true;
			end;
		end;
		else 	self.engineBrakeSoundEnabled = false;
				self.realMotorBrakeFx = self.ebRealMotorBrakeFxBackup;
	end;
end;

function engineBrake:draw()
	if self.isEntered and self.isMotorStarted then
		if self.engineBrake then
			g_currentMission:addHelpButtonText(g_i18n:getText("ENGINEBRAKE_ON"),InputBinding.ENGINEBRAKE);		
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("ENGINEBRAKE_OFF"),InputBinding.ENGINEBRAKE);		
		end;
		if self.ebHiLo then
			g_currentMission:addHelpButtonText(g_i18n:getText("EBHILO_Lo"),InputBinding.EBHILO);
		else
			g_currentMission:addHelpButtonText(g_i18n:getText("EBHILO_Hi"),InputBinding.EBHILO);
		end;
	end;
end;

function engineBrake:mouseEvent(posX, posY, isDown, isUp, button)
end;

function engineBrake:keyEvent(unicode, sym, modifier, isDown)
end;

 function engineBrake:delete()
	 if self.engineBrakeSoundEnabled then
		stopSample(self.engineBrakeSound);
	 end;
 end;
